#!/usr/bin/env python
# -*- coding: utf8 -*-
'''
Utility for running processes on an LSF cluster

Copyright © 2010 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).

'''
__author__ = "Keith Noto <noto@cs.tufts.edu>";
__license__ = "GPL";
__copyright__ = "Copyright © 2010 Keith Noto";
VERSION = __revision__ = "1.0";

import sys,string,subprocess,os,time,hashlib,StringIO;

# Run a UNIX command on an LSF cluster
def submit(cmd, jobname=None, jobdir=".", queue="normal_public", inputfile="/dev/null"):

	# make up a jobname if necessary
	if jobname is None:
		jobname = hashlib.md5(str(cmd)).hexdigest()

	# ensure the existence of jobdir
	try:
		os.makedirs(jobdir)
	except:
		pass

	# decide on filenames
	jobfile = "%s/%s.bsub" % (jobdir,jobname)
	logfile = "%s/%s.log" % (jobdir,jobname)
	outfile = "%s/%s.out" % (jobdir,jobname)
	errfile = "%s/%s.err" % (jobdir,jobname)

	# write job file
	job = open(jobfile,'w')
	job.write("#BSUB -J %s\n" % (jobname))
	job.write("#BSUB -q %s\n" % (queue))
	job.write("#BSUB -i %s\n" % (inputfile))
	job.write("#BSUB -o %s\n" % (logfile))
	job.write("#BSUB -e %s\n" % (errfile))
	job.write("%s 1> %s\n" % (cmd, outfile))
	job.close()

	# launch process (-K option to block until finished)
	p = subprocess.Popen("bsub -K < %s" % (jobfile), shell=True, stdin=None, stdout=None, stderr=subprocess.PIPE, close_fds=True); # shell b/c we want $PATH available
	if p.stderr:
		sys.stderr.write("Error running LSF command:\n\n%s\n\nError is:\n\n%s\n\nKeeping files:\n\n%s\n\n" % (cmd,p.stderr.read(), ', '.join([outfile,errfile,jobfile,logfile])))
		p.stderr.close()
	else:
		map(os.unlink, [outfile,errfile,jobfile,logfile])

	return

