#!/usr/bin/env python
# -*- coding: utf8 -*-
'''

Shared functions

Copyright © 2009 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).

'''

import subprocess,os;

# generate pairs from parallel lists (iterables)
def pairwise(A,B):
	bit = iter(B);
	for a in A:
		b = bit.next();
		yield (a,b);

# pairwise, but raise exception if A,B are different lengths
def gpairs(A,B):

	bit = iter(B)
	for a in A:
		try:
			b = bit.next()
			yield (a,b)
		except StopIteration:
			raise Exception("More A than B:  Asked to generate pairs from sets of unequal size (%s is first unpaired)" % (str(a)))
	try:
		b = bit.next()
		raise Exception("More B than A:  Asked to generate pairs from sets of unequal size (%s is first unpaired)." % (str(b)))
	except StopIteration:
		pass;

# generate index numbers while you iterate
# useful if you need to index into a parallel data structure, but you need to filter 'it'
def iwalk(it):
	i=0
	for x in it:
		yield (i,x)
		i+=1

# create a generator instead of a list to map
def gmap(f,it):
	for x in it:
		yield f(x)

# generate instead of creating a list to filter
def gfilter(f,it):
	for x in it:
		if f(x):
			yield x

# generate items in order from multiple iterators
def gcat(its):	 # "it's..."
	for it in its:
		for x in it:
			yield x

# compute length without necessarily creating a list in memory
def glen(it):
	len=0
	for x in it:
		len+=1
	return len

# for which (index numbers) of the iterable is f(x) true?
def which(f, it):
	return map(lambda (i,x) : i, filter(lambda (i,x) : f(x), iwalk(it)))

# generate unique members 
def guniq(X):
	m = {}
	for x in X:
		if x not in m:
			yield x
		m[x]=True

# reduce X to a list of uniq index numbers
def uniq(X):
	return list(guniq(X))

# return (count,unique member) pairs
def uniqc(X):
	m = {}
	for x in X:
		m[x] = m.get(x,0) + 1;
	return map(lambda (v,c):(c,v), m.iteritems()) # return (count,value) pairs

# Ensure directory's existence
def chkdir(directory):
	if not os.path.exists(directory):
		os.makedirs(directory)
	return directory;


