#!/usr/bin/env python
# -*- coding: utf8 -*-
'''

Utilities for [re]formating data sets

numericize function on loan from frac/supervised/svm.py

Copyright © 2010 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).

'''

import sys,string,math
from util import *

# Numericize:  Deterministically replace nominal features with numerical ones
# For k-valued nominal features, create k binary features.
# Replace missing values in continuous features with a mid-range value
def numericize(X, t):
	for x in X:
		# map x -> x2, where x2 has the same data in numercial features
		assert len(x)==len(t)	# x (data vector) and t (type of each component) are parallel
		yield list(numericize_row(x, t))
def numericize_row(x, t):
	for i in range(len(t)):
		if type(t[i]) is list: # nominal, create binary feature for each possible value
			# can't even keep binary feature as 1 column, since that doesn't handle missing data
			for v in t[i]:  # for each possible value v, 
				yield 1 if x[i]==v else 0	 # (handles missing x[i], too--set all values to zero)
		else: # continuous
			if x[i] is None:
				# missing data, use a mid-range value
				(minv,maxv) = t[i] # types of continuous features is a (min,max) tuple, so this should always work
				yield 0 if minv==-maxv else ((minv+maxv)/2.0) # handles (-inf,+inf)
			else:
				yield x[i] # not missing--just use value itself
def numericize_types(t):
	for t_i in t:
		if t_i is None or type(t_i) is not list:
			yield t_i # type unchanged
		else:
			for v in t_i:
				yield [0,1] # one binary feature per possible feature value

# scale numeric columns (if X has been "numericized" then pass in numericized_types( original types ))
def unitscale(X, t):
	for x in X:
		yield list(unitscale_row(x,t))
def unitscale_row(x,t):
	assert len(x)==len(t)
	for (x_i,t_i) in gpairs(x,t):
		if x_i is None or t_i is None:
			yield x_i # no change
		elif type(t_i) is list:
			yield x_i # no change
		else:
			(minv,maxv) = t_i
			if minv==maxv:
				assert x_i==minv # if minv==maxv, all values must be the same
				yield x_i # no change
			else:
				yield (x_i-minv) / (maxv-minv)
