#!/usr/bin/env python
# -*- coding: utf8 -*-
'''

A fine model, the majority-value predictor 

Copyright © 210 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).

'''
__author__ = "Keith Noto <noto@cs.tufts.edu>";
__license__ = "GPL";
__copyright__ = "Copyright © 2009 Keith Noto";
VERSION = __revision__ = "1.0";

import sys,os,string,StringIO,time,tempfile;
from util import *;

# Predictor:  something that generates predictions
class Predictor:
	def __init__(self, prediction, distribution):
		self.prediction=prediction
		self.distribution=distribution
	def __call__(self,Q):
		for x in Q:
			yield (self.prediction,self.distribution) 
	
# Learner:  something that learns a predictor
class Learner:
	def classifier():
		return False
	def __call__(self, (ftypes,ttype), trainset):
		trainset = filter(lambda (x,y) : y is not None, trainset)	# remove training missing targets
		if type(ttype) is list: # nominal, count each type

			counts = {}
			for (x, y) in trainset:
				counts[y] = counts.get(y,0) + 1
			
			distribution = map( lambda label : counts[label] / float(len(trainset)) , ttype )
			majority = ttype[ distribution.index(max(distribution)) ]

			return Predictor(majority,distribution)

		else: # continuous, use mean

			return Predictor( float(sum(map(lambda (x,y):y, trainset))) / len(trainset), None)

