#!/usr/bin/env pyth/n
# -*- coding: utf8 -*-
'''

Learning with Classification/Regression Trees (using WEKA).  

This class uses the weka module and updates the Learner/Predictor in that module to be specific to trees.

This is part of FRaC.

Copyright © 2010 Keith Noto.  Disributed under Gnu General Public License (see http://www.gnu.org/licenses/gpl.html).

'''
__author__ = "Keith Noto <noto@cs.tufts.edu>";
__license__ = "GPL";
__copyright__ = "Copyright © 2010 Keith Noto";
VERSION = __revision__ = "1.0";

import sys,os,string
import parrot,lsf
import weka
from util import *;

verbose = nullstream = open(os.devnull, 'w')
J48="J48"
REPTree="REPTree"
algorithms = [J48,REPTree] # define tree algorithms

class Learner(weka.Learner):

	def classifier():
		return False # This tree is a regression model, too

	def __init__(self, clean=True):
		# As a WEKA learner, set training command tcmd and prediction command pcmd when I have more info
		weka.Learner.__init__(self, tcmd=None, pcmd=None, clean=clean) 

	# overwrite weka.Learner.__call__ to set up options that depend on the
	# training data
	def __call__(self, (ftypes,ttype), T):

		# check trainset...
		if not filter(lambda (x,y) : y is not None, T):
			raise(Exception("Call to Tree Learner with zero labeled training examples"))
			# one could return parrot.Predictor(ttype[0]) arbitrarily 
		
		# Choose model type based on target type
		algo = J48 if type(ttype) is list else REPTree
		
		# Set pruning options (trees sometimes crash even with default
		# parameters, so we have to make sure the options are valid)
		#
		# REPTree does pruning by default (-P to turn it off; -N option
		# means number of folds--must be <= number of examples)
		#
		# J48 does not do pruning by default, requires -R option (and then
		# -N is the number of folds)

		if len(T) > 3:
			options = " -R -N 3 " if algo in [J48] else " -N 3 "; # 3 is the default.
		else:	# good luck with your 3 training examples...
			options = " -P " if algo in [REPTree] else ""	# 2 (or 1) examples; turn pruning off (we just want the options to be compatible with the input so the program doesn't crash)
		
		self.tcmd = "weka.classifiers.trees.%s %s" % (algo,options)
		self.pcmd = "weka.classifiers.trees.%s" % (algo)
	
		# now that training/prediction commands are set, call weka.Learner's call method
		return weka.Learner.__call__(self, (ftypes,ttype), T) # return weka.Predictor

