#!/bin/bash

DATASETS="haberman iris balance-scale blood-transfusion hayes-roth tae
mammographic-masses car acute nursery ecoli abalone pima-indians-diabetes cmc
yeast tic-tac-toe glass echocardiogram wine credit-screening voting-records zoo
hepatitis image statlog mushroom parkinsons horse-colic ionosphere dermatology
chess annealing audiology libras"

BIG_DATASETS="magic poker page-blocks adult letter-recognition
breast-cancer-wisconsin cylinder-bands connect-4 spambase ozone"

HUGE_DATASETS="arrhythmia secom internet_ads"

MISSING_DATASETS="echocardiogram hepatitis mammographic-masses voting-records
horse-colic credit-screening dermatology audiology cylinder-bands annealing
arrhythmia mushroom ozone adult secom internet_ads"

DECOY_TABLE_DATASETS="mushroom zoo iris car balance-scale acute tic-tac-toe" # Everybody does well on these
DECOY_GRAPH_DATASETS="zoo iris balance-scale acute" # We'll make drawn-out graphs for these datasets

launches=0
LAUNCHLIMIT=20   # choose based on the number of jobs each launch will create 
GROUPSIZE=12     # choose based on the time per feature 

function main { 

	for seed in 131 # `enum 125 150`
	do
	for task in  ""  #  "-u"
	do
	for i in  1 2 3 4 5 6 7 8 9 10 15 20 30 40 50 60 70 80 90 100 200 500 1000
		# 0
		# 10 100 
	do
			
		if [ "${task}" == "-u" ]; then
			TASK_FLAGS="-L -O -R --cfa-rbfsvm -N --cfa-linearsvm -T --cfa-c45"
		else
			TASK_FLAGS="-L -O -B -R --cfa-rbfsvm -N --cfa-linearsvm -T --cfa-c45 --cfa-nbc --cfa-ripper --q5c45 --q5rbfsvm --q5linearsvm --q5nbc --q5ripper" 
		fi
		
		for flag in    "-L" "-O" "-N" "-R" "-T" "--cfa-rbfsvm" "--cfa-linearsvm" "--cfa-c45" "-L --Tbag 100" # (for decoy experiments)
			# ${TASK_FLAGS}
		do

			for d in  $DECOY_GRAPH_DATASETS 
				# $DATASETS $HUGE_DATASETS $BIG_DATASETS
				# $DECOY_TABLE_DATASETS 
			do

				launch "${task}" "${d}" "${seed}" "${i}" "${flag}" long_public

				if [ ${launches} -ge ${LAUNCHLIMIT} ]; then 
					echo "# (Reached launch limit of ${LAUNCHLIMIT})"
					exit
				fi

			done
		
		done


	done
	done
	done

}


function flag2file {
    flag=$1
	if [ "${flag}" == "-O" ];              then echo -ne "one-class-svm.rbf"; fi;
	
	if [ "${flag}" == "-L" ];              then echo -ne "lof.10-100"; fi;
	if [ "${flag}" == "-L --Tbag 100" ];   then echo -ne "lof.10-100.100bags"; fi;
	
	if [ "${flag}" == "-J" ];              then echo -ne "cof.10-100"; fi;
	if [ "${flag}" == "-J --Tbag 100" ];   then echo -ne "cof.10-100.100bags"; fi;
	
	if [ "${flag}" == "--cfa-rbfsvm" ];    then echo -ne "cfa2.rbfsvm.apx"; fi;
	if [ "${flag}" == "--cfa-c45" ];       then echo -ne "cfa2.tree.apx"; fi;
	if [ "${flag}" == "--cfa-linearsvm" ]; then echo -ne "cfa2.linearsvm.apx"; fi;
	if [ "${flag}" == "--cfa-ripper" ];    then echo -ne "cfa2.ripper.apx"; fi;
	if [ "${flag}" == "--cfa-nbc" ];       then echo -ne "cfa2.nbc.apx"; fi;

	if [ "${flag}" == "--q5rbfsvm" ];    then echo -ne "frac.q5.2.rbfsvm"; fi;
	if [ "${flag}" == "--q5c45" ];       then echo -ne "frac.q5.2.tree"; fi;
	if [ "${flag}" == "--q5linearsvm" ]; then echo -ne "frac.q5.2.linearsvm"; fi;
	if [ "${flag}" == "--q5ripper" ];    then echo -ne "frac.q5.2.ripper"; fi;
	if [ "${flag}" == "--q5nbc" ];       then echo -ne "frac.q5.2.nbc"; fi;
	
	if [ "${flag}" == "-N" ];              then echo -ne "frac.linearsvm"; fi;
	if [ "${flag}" == "-R" ];              then echo -ne "frac.rbfsvm"; fi;
	if [ "${flag}" == "-T" ];              then echo -ne "frac.tree"; fi;
}

function d2f {
	d=$1
	features=0
	if [ "${d}" == "mushroom" ]; then                 features=22; fi
	if [ "${d}" == "zoo" ]; then                      features=17; fi
	if [ "${d}" == "iris" ]; then                     features=4; fi
	if [ "${d}" == "car" ]; then                      features=6; fi
	if [ "${d}" == "balance-scale" ]; then            features=4; fi
	if [ "${d}" == "acute" ]; then                    features=7; fi
	if [ "${d}" == "tic-tac-toe" ]; then              features=9; fi
	if [ "${d}" == "magic" ]; then                    features=10; fi
	if [ "${d}" == "poker" ]; then                    features=10; fi
	if [ "${d}" == "page-blocks" ]; then              features=10; fi
	if [ "${d}" == "adult" ]; then                    features=14; fi
	if [ "${d}" == "letter-recognition" ]; then       features=16; fi
	if [ "${d}" == "breast-cancer-wisconsin" ]; then  features=31; fi
	if [ "${d}" == "cylinder-bands" ]; then           features=39; fi
	if [ "${d}" == "connect-4" ]; then                features=42; fi
	if [ "${d}" == "spambase" ]; then                 features=57; fi
	if [ "${d}" == "ozone" ]; then                    features=73; fi
	if [ "${d}" == "arrhythmia" ]; then               features=262; fi
	if [ "${d}" == "secom" ]; then                    features=475; fi
	if [ "${d}" == "internet_ads" ]; then             features=1558; fi
	echo -ne "${features}"
}

function launch {

	task=$1
	d=$2
	seed=$3
	i=$4
	flag="$5"
	queue=$6
	features=`d2f "${d}"`
	let features=${features}+${i}

	file=`flag2file "${flag}"`
	rdir="results.semi-supervised"
	if [ "${task}" == "-u" ]; then rdir="results.unsupervised"; fi;
	full="${rdir}/${seed}/${d}/${file}"
	if [ "${i}" -gt 0 ]; then full="${rdir}/${seed}/${d}.i${i}/${file}"; fi;

	if [ -s "${full}" ]; then
		
		# echo "# Output file exists:  ${full}" >&2 
		echo -ne ""; # noop

	else

		echo "# Output file DOES NOT EXIST:  ${full}" >&2

		if [ "`hostname`" == "fatman.eecs.tufts.edu" ]; then
			time run metadata -d ${d} --seed ${seed} ${task} -i ${i} ${flag} 
		else

			if [ ${features} -eq 0 -o "${flag}" == "-L" -o "${flag}" == "-O" -o "${flag}" == "-B" -o "${flag}" == "-L --Tbag 100" ]; then 
				echo -ne "# LAUNCH ${d} (all features), seed ${seed}, i ${i}, flag(s): ${flag}\n"
				jobs/x1 ${d} ${seed} ${queue} "${task} -i ${i} ${flag}" | bsub
			else
				if [ ${features} -gt 1000 -a ${GROUPSIZE} -eq 1 ]; then groupsize=2; else groupsize=${GROUPSIZE}; fi;
					echo -ne "# LAUNCH ${d} (${features} features), seed ${seed}, i ${i}, flag(s): ${flag}\n"
				jobs/xg ${d} $seed ${queue} ${features} ${groupsize} "${task} -i ${i} ${flag}" | bsub
				let launches=${launches}+1 # (optionally can only count as a launch if it's a multi-feature job)
			fi;
			

		fi;
	fi;

}

main

